@@ -13,7 +13,7 @@ fit_hbd_psr_on_grid = function(	tree,
 								guess_PSR			= NULL,		# initial guess for the PSR. Either NULL (an initial guess will be computed automatically), or a single numeric (guessing a constant PSR at all ages) or a numeric vector of size NG specifying an initial guess for the PSR at each age-grid point (can include NAs)
 								fixed_PSR			= NULL,		# optional fixed PSR values, on one or more of the age grid points. Either NULL (none of the PSRs are fixed), or a single scalar (all PSRs are fixed) or a numeric vector of size NG (some or all PSRs are fixed, can include NAs).
 								splines_degree		= 1,		# integer, either 1 or 2 or 3, specifying the degree for the splines defined by the PSR on the age grid.
-								condition			= "stem",	# one of "crown" or "stem", specifying whether to condition the likelihood on the survival of the stem group or the crown group. It is recommended to use "stem" when oldest_age>root_age, and "crown" when oldest_age==root_age. This argument is similar to the "cond" argument in the R function RPANDA::likelihood_bd. Note that "crown" really only makes sense when oldest_age==root_age.
+								condition			= "auto",	# either "crown" or "stem" or "auto", specifying whether to condition the likelihood on the survival of the stem group or the crown group. It is recommended to use "stem" when oldest_age>root_age, and "crown" when oldest_age==root_age. This argument is similar to the "cond" argument in the R function RPANDA::likelihood_bd. Note that "crown" really only makes sense when oldest_age==root_age.
 								relative_dt			= 1e-3,		# maximum relative time step allowed for integration. Smaller values increase the accuracy of the computed likelihoods, but increase computation time. Typical values are 0.0001-0.001. The default is usually sufficient.
 								Ntrials					= 1,
 								Nbootstraps				= 0,		# (integer) optional number of parametric-bootstrap samples (random trees generated using the fitted PSR) for estimating confidence intervals of fitted parameters. If 0, no parametric bootstrapping is performed. Typical values are 10-100.
@@ -31,8 +31,11 @@ fit_hbd_psr_on_grid = function(	tree,
 	if(age0<0) return(list(success = FALSE, error="age0 must be non-negative"));
 	root_age = get_tree_span(tree)$max_distance
 	if(is.null(oldest_age)) oldest_age = root_age;
-	if(root_age<age0) return(list(success=FALSE, error=sprintf("age0 is older than the root age (%g)",root_age)));
+	if(root_age<age0) return(list(success=FALSE, error=sprintf("age0 (%g) is older than the root age (%g)",age0,root_age)));
+	if(oldest_age<age0) return(list(success=FALSE, error=sprintf("age0 (%g) is older than the oldest considered age (%g)",age0,oldest_age)));
 	if((!is.null(age_grid)) && (length(age_grid)>1) && ((age_grid[1]>age0) || (tail(age_grid,1)<oldest_age))) return(list(success = FALSE, error=sprintf("Provided age-grid range (%g - %g) does not cover entire required age range (%g - %g)",age_grid[1],tail(age_grid,1),age0,oldest_age)));
+	if(!(condition %in% c("crown","stem","auto"))) return(list(success = FALSE, error = sprintf("Invalid condition '%s': Extected 'stem', 'crown' or 'auto'.",condition)));
+	if(condition=="auto") condition = (if(abs(oldest_age-root_age)<=1e-10*root_age) "crown" else "stem")
 
 	# trim tree at age0 if needed, while shifting time for the subsequent analyses (i.e. new ages will start counting at age0)
 	if(age0>0){
@@ -96,7 +99,8 @@ fit_hbd_psr_on_grid = function(	tree,
 	
 	# guess reasonable start params, if not provided
 	default_guess_PSR = mean(lineage_counter$relative_slopes); # a reasonable guesstimate for the PSR is the relative LTT-slope
-	guess_PSR[is.na(guess_PSR)] = default_guess_PSR;
+	if(default_guess_PSR==0) default_guess_PSR=log(LTT0)/root_age
+	guess_PSR[!is.finite(guess_PSR)] = default_guess_PSR;
 	guess_PSR = pmin(max_PSR, pmax(min_PSR, guess_PSR)); # make sure initial guess is within the imposed bounds
 	
 	# determine which parameters are to be fitted
@@ -110,7 +114,7 @@ fit_hbd_psr_on_grid = function(	tree,
 	NFP					= length(fitted_params);
 	
 	# determine typical parameter scales
-	scale_PSR = abs(guess_PSR); scale_PSR[scale_PSR==0] = mean(scale_PSR);
+	scale_PSR = abs(guess_PSR); scale_PSR[scale_PSR==0] = mean(scale_PSR); scale_PSR[scale_PSR==0]=default_guess_PSR;
 	param_scales = c(rep(scale_PSR,times=NG));
 
 
@@ -154,10 +158,14 @@ fit_hbd_psr_on_grid = function(	tree,
 		# randomly choose start values for fitted params
 		start_values = guess_param_values[fitted_params]
 		if(trial>1){
-			boxed   = which(!(is.infinite(lower_bounds) | is.infinite(upper_bounds))); # determine fitted params that are boxed, i.e. constrained to within finite lower & upper bounds
-			unboxed = completement(NFP, boxed);
-			if(length(boxed)>0) start_values[boxed] = lower_bounds[boxed] + (upper_bounds[boxed]-lower_bounds[boxed]) * runif(n=length(boxed),min=0,max=1)
-			if(length(unboxed)>0) start_values[unboxed]	= 10**runif(n=length(unboxed), min=-2, max=2) * start_values[unboxed]
+			boxed_left	= which((!is.infinite(lower_bounds)) & is.infinite(upper_bounds))
+			boxed_right	= which((!is.infinite(upper_bounds)) & is.infinite(lower_bounds))
+			boxed_dual  = which(!(is.infinite(lower_bounds) | is.infinite(upper_bounds))); # determine fitted params that are boxed, i.e. constrained to within finite lower & upper bounds
+			unboxed 	= which(is.infinite(lower_bounds) & is.infinite(upper_bounds))
+			if(length(boxed_dual)>0) 	start_values[boxed_dual] = lower_bounds[boxed_dual] + (upper_bounds[boxed_dual]-lower_bounds[boxed_dual]) * runif(n=length(boxed_dual),min=0,max=1)
+			if(length(unboxed)>0) 	 	start_values[unboxed]	 = 10**runif(n=length(unboxed), min=-2, max=2) * start_values[unboxed]
+			if(length(boxed_left)>0) 	start_values[boxed_left] = sapply(boxed_left, FUN=function(fp) random_semiboxed_left(lower_bound=lower_bounds[fp], default=start_values[fp], typical_scale=scales[fp], orders_of_magnitude=4))
+			if(length(boxed_right)>0) 	start_values[boxed_right]= sapply(boxed_right, FUN=function(fp) -random_semiboxed_left(lower_bound=-upper_bounds[fp], default=-start_values[fp], typical_scale=scales[fp], orders_of_magnitude=4))
 		}
 		start_values = pmax(lower_bounds,pmin(upper_bounds,start_values))
 		# run fit
@@ -236,11 +244,11 @@ fit_hbd_psr_on_grid = function(	tree,
 			# simulate model with fitted parameters
 			if(verbose) cat(sprintf("%s  Bootstrap #%d..\n",verbose_prefix,b))
 			bootstrap = castor::generate_tree_hbd_reverse(	Ntips			= original_Ntips,
-																crown_age		= root_age,
-																age_grid		= sim_age_grid, 
-																PSR				= sim_PSR,
-																splines_degree	= splines_degree,
-																relative_dt		= relative_dt)
+															crown_age		= root_age,
+															age_grid		= sim_age_grid, 
+															PSR				= sim_PSR,
+															splines_degree	= splines_degree,
+															relative_dt		= relative_dt)
 			if(!bootstrap$success) return(list(success=FALSE, error=sprintf("Bootstrapping failed: Could not generate tree for the fitted PSR: %s",bootstrap$error), age_grid=age_grid, fitted_PSR=fitted_PSR, loglikelihood=loglikelihood));
 			bootstrap_tree = bootstrap$trees[[1]]
 
