@@ -28,7 +28,7 @@ fit_musse = function(	tree,
 						proxy_map				= NULL,			# optional 1D integer vector of size Nstates, mapping states to proxy-states. Hence, proxy_map[s] is an integer in 1:NPstates, specifying which proxy-state the state s belongs to. Only relevant if NPstates!=NULL and NPstates!=Nstates.
 						state_names				= NULL,			# optional 1D character vector of size Nstates, specifying a name/description for each state. This does not influence any of the calculations, and is merely used to add human-readable names (rather than integers) to the returned vectors/matrices.
 						tip_pstates				= NULL,			# 1D integer array of size Ntips, listing integers in 1:NPstates, specifying the proxy state at each tip. Can also be NULL. May include NA (unknown tip states).
-						tip_priors 				= NULL, 		# 2D numerical array of either size Ntips x Nstates or size Ntips x NPstates, listing the prior likelihood of each state (or proxy-state) at each tip. Specifically, tip_priors[i,s] is the likelihood of observing the data, if tip i had state (or proxy-state) s. Can be provided alternatively to tip_pstates.
+						tip_priors 				= NULL, 		# 2D numerical array of either size Ntips x Nstates or size Ntips x NPstates, listing the prior likelihood at each state (or proxy-state) at each tip. Specifically, tip_priors[i,s] is the likelihood of observing the data at tip i, if tip i had state (or proxy-state) s. Can be provided alternatively to tip_pstates.
 						sampling_fractions		= 1,			# optional numerical vector of size NPstates, indicating the sampling fractions (fraction of species included as tips in the tree) conditional upon each species' proxy state. This can be used to incorporate detection biases for species, depending on their proxy state. Can also be NULL or a single number (in which case sampling fractions are assumed to be independent of proxy state).
 						reveal_fractions		= 1,			# optional numerical vector of size NPstates, indicating the resolved fractions (fraction of tree-tips with revealed, i.e. non hidden, state) conditional upon each tip's proxy state. Hence, reveal_fractions[s] is the probability that a species with proxy state s will have revealed state, conditional upon being included in the tree. This can be used to incorporate reveal biases for tips, depending on their proxy state. Can also be NULL or a single number (in which case reveal fractions are assumed to be independent of proxy state).
 						transition_rate_model	= "ARD",		# either "ER" or "SYM" or "ARD" or "SUEDE" or "SRD" or a 2D integer matrix of size Nstates x Nstates mapping entries of the transition matrix to a set of independent transition-rate parameters. The format and interpretation is the same as for index_matrix generated by the function get_transition_index_matrix(..).
@@ -40,8 +40,9 @@ fit_musse = function(	tree,
 						first_guess				= NULL,			# either NULL, or a named list containing optional entries 'transition_matrix', 'birth_rates' and/or 'death_rates'. Each entry specifies start values (first guess) for the particular model parameter. Some start values may include NA, corresponding to unknowns which are to be guessed internally. Note that any provided lower & upper bounds overrule the first_guess, i.e. first_guess is forced to be within any specified bounds.
 						lower					= NULL,			# either NULL, or a named list containing optional entries 'transition_matrix', 'birth_rates' and/or 'death_rates'. Each entry specifies lower bounds for model parameters (only relevant if a parameter is not fixed). Parameters not listed in lower[] are assumed not explicitly bounded (except for >=0 when reasonable). To keep a parameter unbounded, you can also use NA, -Inf or NaN.
 						upper					= NULL,			# either NULL, or a named list containing optional entries 'transition_matrix', 'birth_rates' and/or 'death_rates'. Each entry specifies upper bounds for model parameters (only relevant if a parameter is not fixed). Parameters not listed in upper[] are assumed not explicitly bounded. To keep a parameter unbounded, you can also use NA, +Inf or NaN.
-						root_prior 				= "likelihoods",# can be 'flat', 'empirical' or 'likelihoods' or a numeric vector of size Nstates. Specifies the weights for averaging the root's state likelihoods in order to obtain an overall model likelihood. 'empirical' means the root's prior is set to the proportions of (estimated) extant species in each state (correcting for sampling_fractions and reveal_fractions). 'likelihoods' means that the computed state-likelihoods of the root (i.e. D_s) are used, after normalizing to obtain an probability distribution; this is the approach used in hisse::hisse if root.p=NULL, and the approach in diversitree when root=ROOT.OBS.
-						root_conditioning		= "none",		# can be 'none', 'madfitz' or 'herr_als', specifying how to condition the state-likelihoods at the root prior to averaging. "none" (no conditioning) uses the raw state-likelihoods, as originally described by Maddison (2007). "madfitz" and "herr_als" are the options implemented in the hisse package, conditioning the root's state-likelihoods based on the birth-rates and the computed extinction probability (after or before averaging, respectively). For typical use cases, we suspect that this option does not matter much.
+						root_prior 				= "auto",		# can be 'flat', 'empirical', 'likelihoods', 'max_likelihood', 'auto' or a numeric vector of size Nstates. Specifies the weights for averaging the root's state likelihoods in order to obtain an overall model likelihood. 'empirical' means the root's prior is set to the proportions of (estimated) extant species in each state (correcting for sampling_fractions and reveal_fractions). 'likelihoods' means that the computed state-likelihoods of the root (i.e. D_s) are used, after normalizing to obtain an probability distribution; this is the approach used in hisse::hisse if root.p=NULL, and the approach in diversitree when root=ROOT.OBS.
+						root_conditioning		= "auto",		# can be 'none', 'madfitz', 'herr_als', 'stem', 'crown' or 'auto', specifying how to condition the state-likelihoods at the root prior to averaging. "none" (no conditioning) uses the raw state-likelihoods, as originally described by Maddison (2007). "madfitz" and "herr_als" are the options implemented in the hisse package, conditioning the root's state-likelihoods based on the birth-rates and the computed extinction probability (after or before averaging, respectively). For typical use cases, we suspect that this option does not matter much.
+						oldest_age				= NULL,			# oldest age to consider. Typically this will be <=root_age. If NULL, this will be set to root_age.
 						Ntrials 				= 1,			# (integer) number of trials (starting points) for fitting the model. If 0, then no fitting is performed, and only the a first-guess (start params) is evaluated and returned.
 						optim_algorithm 		= "subplex",	# either "optim", "nlminb" or "subplex". What algorithm to use for fitting.
 						optim_max_iterations	= 10000,		# maximum number of iterations of the optimization algorithm (per trial)
@@ -71,13 +72,16 @@ fit_musse = function(	tree,
 	death_rate_indices		= get_rate_index_vector(Nstates, death_rate_model)$index_vector
 	is_hisse_model			= !(is.null(NPstates) || (NPstates==0) || (NPstates==Nstates))
 	if(!is_hisse_model) NPstates = Nstates
+	root_age = get_tree_span(tree)$max_distance
+	if(is.null(oldest_age)) oldest_age = root_age
 			
 	# check validity of input variables, and determine known tips
 	if(verbose) cat(sprintf("%sChecking input variables..\n",verbose_prefix))
 	if(!(optim_algorithm %in% c("subplex", "optim", "nlminb"))) stop(sprintf("ERROR: Invalid optimization algorithm '%s'",optim_algorithm))
 	if((!is.null(tip_pstates)) && (!is.null(tip_priors))) stop("ERROR: tip_pstates and tip_priors are both non-NULL, but exactly one of them should be NULL")
 	else if(is.null(tip_pstates) && is.null(tip_priors))  stop("ERROR: tip_pstates and tip_priors are both NULL, but exactly one of them should be non-NULL")
-	if(! (root_conditioning %in% c("none","madfitz","herr_als"))) stop("ERROR: root_conditioning must be one of 'none', 'madfitz' or 'herr_als'")
+	if(! (root_conditioning %in% c("none","madfitz","herr_als","stem","crown","auto"))) stop("ERROR: root_conditioning must be one of 'none', 'madfitz', 'herr_als', 'stem', 'crown' or 'auto'")
+	if(root_conditioning=="auto") root_conditioning = (if(abs(oldest_age-root_age)<=1e-10*root_age) "crown" else "stem");
 	if(is.null(Nstates) || is.na(Nstates) || (Nstates==0)) stop("ERROR: Nstates must be a strictly positive integer")
 	if((!is.null(state_names)) && (length(state_names)!=Nstates)) stop(sprintf("ERROR: Number of provided state_names (%d) differs from Nstates (%d)",length(state_names),Nstates))
 	if(is_hisse_model && is.null(proxy_map)) stop("ERROR: Missing proxy_map, needed for HiSSE model")
@@ -198,13 +202,28 @@ fit_musse = function(	tree,
 	else death_rates = rep(NA, Nstates)
 	
     # figure out prior distribution for root if needed
+    if(root_prior[1]=="auto"){
+    	if(oldest_age>=(1+1e-10)*root_age){
+    		root_prior = "likelihoods"
+    	}else{
+    		root_prior = "max_likelihood"
+    	}
+    }
     if(root_prior[1]=="flat"){
+    	root_prior_type = "custom"
     	root_prior_probabilities = rep(1.0/Nstates, times=Nstates);
     }else if(root_prior[1]=="empirical"){
     	# if is_hisse_model, then the empirical probability of each proxy-state is subdivided equally among all states represented by that proxy state
+    	root_prior_type = "custom"
 		root_prior_probabilities = ((Nspecies_per_pstate/sum(Nspecies_per_pstate))/Nstates_per_pstate)[proxy_map];
 	}else if(root_prior[1]=="likelihoods"){
-		root_prior_probabilities = numeric(0) # root prior will be set to the computed state-likelihoods at the root, and thus depend on the particular model parameters. 
+		# root prior will be set to the computed state-likelihoods at the root, and thus depend on the particular model parameters. 
+    	root_prior_type = "likelihoods"
+		root_prior_probabilities = numeric(0)
+	}else if(root_prior[1]=="max_likelihood"){
+		# root prior will be set to a Dirac distribution, localized at the state with highest likelihood
+    	root_prior_type = "max_likelihood"
+		root_prior_probabilities = numeric(0)
 	}else if((length(root_prior)==1) && (is.character(root_prior[[1]]))){
 		stop(sprintf("ERROR: Unknown root_prior '%s'",root_prior[[1]]))
 	}else{
@@ -214,6 +233,7 @@ fit_musse = function(	tree,
 			if(any(root_prior<0)) stop(sprintf("ERROR: root_prior contains negative values (down to %g)",min(root_prior)))
 			if(abs(1.0-sum(root_prior))>1e-6) stop(sprintf("ERROR: Entries in root prior do not sum up to 1 (sum=%.10g)",sum(root_prior)))
 		}
+    	root_prior_type = "custom"
 		root_prior_probabilities = root_prior
 	}
 	
@@ -353,12 +373,15 @@ fit_musse = function(	tree,
 	# now proceed with the fitting trials
 
 	if(verbose) cat(sprintf("%sPreparing fitting of full model..\n",verbose_prefix))
+
+	# pre-calculate node ages, used in loglikelihood function
+	node_ages = get_all_node_depths(tree);			
 			
 	# determine typical parameter scales
 	param_scales = lapply(first_guess,function(x) abs(x))
 	if(all(param_scales$birth_rates==0)){
 		# all birth-rate scales are zero, so determine scales based on a simple Yule model
-		param_scales$birth_rates[] = log(Ntips)/max(node_ages);
+		param_scales$birth_rates[] = log(Ntips)/root_age;
 	}else{
 		# some (but not all) birth-rate scales are zero, so determine scale based on mean birth-rate scale
 		param_scales$birth_rates[param_scales$birth_rates==0] = mean(param_scales$birth_rates)
@@ -378,10 +401,7 @@ fit_musse = function(	tree,
 		param_scales$transition_matrix[param_scales$transition_matrix==0] = 0.1*mean(param_scales$transition_matrix)
 	}
 	param_scales = compress_params(param_scales$transition_matrix, param_scales$birth_rates, param_scales$death_rates, Nstates, transition_indices, birth_rate_indices, death_rate_indices);
-	
-	# pre-calculate node ages, used in loglikelihood function
-	node_ages = get_all_node_depths(tree);			
-					
+						
 	# define objective function to be minimized (negated log-likelihood)
 	# the input to the objective function must be scaled and free (non-fixed) independent parametes
 	objective_function = function(fparam_values){
@@ -391,10 +411,11 @@ fit_musse = function(	tree,
 		if(any(fparam_values<param_mins[fitted_params]) || any(fparam_values>param_maxs[fitted_params])) return(Inf)
 		param_values 	= provided_param_values; param_values[fitted_params] = fparam_values; # merge fixed & fitted parameter values
 		param_values 	= uncompress_params(param_values, Nstates, transition_indices, birth_rate_indices, death_rate_indices, NULL)
-		results = get_MuSSE_loglikelihood_CPP(	Ntips 							= Ntips,
+		results = tryCatch({ get_MuSSE_loglikelihood_CPP(	Ntips 							= Ntips,
 												Nnodes							= Nnodes,
 												Nedges							= Nedges,
 												Nstates							= Nstates,
+												oldest_age						= oldest_age,
 												tree_edge 						= as.vector(t(tree$edge))-1,	# flatten in row-major format and make indices 0-based,
 												node_ages 						= node_ages,
 												transition_rates 				= as.vector(t(param_values$transition_matrix)),	# flatten in row-major format
@@ -402,6 +423,7 @@ fit_musse = function(	tree,
 												extinction_rates 				= param_values$death_rates,
 												initial_D_per_tip 				= as.vector(t(tip_priors)), 		# flatten in row-major format
 												initial_E_per_state				= initial_extinction_probabilities,
+												root_prior_type					= root_prior_type,
 												root_prior 						= root_prior_probabilities,
 												root_conditioning				= root_conditioning,
 												include_ancestral_likelihoods 	= FALSE,
@@ -410,7 +432,8 @@ fit_musse = function(	tree,
 												relative_ODE_step				= relative_ODE_step,
 												E_value_step					= E_value_step,
 												D_temporal_resolution			= D_temporal_resolution,
-												runtime_out_seconds				= max_model_runtime);
+												runtime_out_seconds				= max_model_runtime)
+							}, error = function(e){ list(loglikelihood=NaN, success=FALSE) })
 		loglikelihood = if((!results$success) || is.na(results$loglikelihood) || is.nan(results$loglikelihood)) -Inf else results$loglikelihood;
 		return(-loglikelihood);
 	}
@@ -420,8 +443,9 @@ fit_musse = function(	tree,
 	scaled_param_mins = scale_params(param_mins,param_scales,param_mins);
 	scaled_param_maxs = scale_params(param_maxs,param_scales,param_mins);
 	fit_single_trial = function(trial){
-		initial_param_values = if(trial==1) first_guess_compr else first_guess_compr * pmin(1e6,pmax(1e-6,2**runif(n=NIP, min=-sqrt(Ntrials), max=sqrt(Ntrials))));
-		initial_param_values = pmin(pmax(initial_param_values, param_mins), param_maxs) # make sure randomized start params are within bounds
+		#initial_param_values = if(trial==1) first_guess_compr else first_guess_compr * pmin(1e6,pmax(1e-6,2**runif(n=NIP, min=-sqrt(Ntrials), max=sqrt(Ntrials))));
+		#initial_param_values = pmin(pmax(initial_param_values, param_mins), param_maxs) # make sure randomized start params are within bounds
+		initial_param_values = if(trial==1) first_guess_compr else get_random_params(defaults=first_guess_compr, lower_bounds=param_mins, upper_bounds=param_maxs, scales=param_scales, orders_of_magnitude=max(1,min(6,log2(10.0)*sqrt(Ntrials))))
 		scaled_initial_param_values = scale_params(initial_param_values,param_scales,param_mins)
 		if(optim_algorithm == "optim"){
 			fit = stats::optim(	scaled_initial_param_values[fitted_params], 
@@ -538,6 +562,7 @@ fit_musse = function(	tree,
 										Nnodes							= Nnodes,
 										Nedges							= Nedges,
 										Nstates							= Nstates,
+										oldest_age						= oldest_age,
 										tree_edge 						= as.vector(t(tree$edge))-1,	# flatten in row-major format and make indices 0-based,
 										node_ages 						= node_ages,
 										transition_rates 				= as.vector(t(best_param_values$transition_matrix)),	# flatten in row-major format
@@ -545,6 +570,7 @@ fit_musse = function(	tree,
 										extinction_rates 				= best_param_values$death_rates,
 										initial_D_per_tip 				= as.vector(t(tip_priors)), 		# flatten in row-major format
 										initial_E_per_state				= initial_extinction_probabilities,
+										root_prior_type					= root_prior_type,
 										root_prior 						= root_prior_probabilities,
 										root_conditioning				= root_conditioning,
 										include_ancestral_likelihoods 	= include_ancestral_likelihoods,
@@ -559,8 +585,13 @@ fit_musse = function(	tree,
 		return(return_value_on_failure);
 	}
 	loglikelihood = final$loglikelihood;
-	if(include_ancestral_likelihoods) ancestral_likelihoods = matrix(final$ancestral_likelihoods, ncol=Nstates, byrow=TRUE, dimnames=list(tree$tip.label,NULL)) # unflatten
-	final$ML_root_state = final$ML_root_state + 1 # make index 1-based
+	if(include_ancestral_likelihoods){
+		ancestral_likelihoods = matrix(final$ancestral_likelihoods, ncol=Nstates, byrow=TRUE, dimnames=list(tree$tip.label,NULL)) # unflatten
+		if(oldest_age<root_age) ancestral_likelihoods[node_ages>oldest_age,] = NA; # state likelihoods are not calculated for nodes older than oldest_age
+	}
+	final$ML_subroot_states	= final$ML_subroot_states + 1 # make index 1-based
+	final$ML_substem_states	= final$ML_substem_states + 1 # make index 1-based
+	final$subroots 			= final$subroots + 1
 	
 	
 	# estimate confidence intervals if needed, using parametric bootstrapping
@@ -577,7 +608,6 @@ fit_musse = function(	tree,
 										NPstates 				= NPstates,
 										proxy_map 				= proxy_map,
 										parameters 				= best_param_values,
-										start_state 			= final$ML_root_state,
 										max_tips 				= Ntips/overall_rarefaction,
 										sampling_fractions		= sampling_fractions,
 										reveal_fractions		= reveal_fractions,
@@ -604,6 +634,7 @@ fit_musse = function(	tree,
 							upper					= upper,
 							root_prior 				= root_prior,
 							root_conditioning		= root_conditioning,
+							oldest_age				= oldest_age,
 							Ntrials 				= Ntrials_per_bootstrap,
 							optim_algorithm 		= optim_algorithm,
 							optim_max_iterations	= optim_max_iterations,
@@ -645,6 +676,7 @@ fit_musse = function(	tree,
 	return(list(success					= TRUE, 
 				Nstates					= Nstates,
 				NPstates				= NPstates,
+				root_prior				= root_prior,
 				parameters				= best_param_values, 
 				start_parameters		= first_guess,
 				loglikelihood			= loglikelihood,
@@ -653,7 +685,9 @@ fit_musse = function(	tree,
 				Nevaluations			= Nevaluations, # may be NA, depending on the optimization algorithm
 				converged				= converged,
 				warnings				= (if(length(final$warnings)>0) final$warnings else NULL),
-				ML_root_state			= final$ML_root_state, # maximum-likelihood estimate of the root state, based on the computed state-likelihoods
+				subroots				= final$subroots, # clade indices of subtree roots
+				ML_subroot_states		= final$ML_subroot_states, # maximum-likelihood estimate of the state at each subtree's root, based on the computed state-likelihoods
+				ML_substem_states		= final$ML_substem_states, # maximum-likelihood estimate of the state at each subtree's stem, based on the computed state-likelihoods
 				standard_errors			= (if(Nbootstraps>0) standard_errors else NULL),
 				CI50lower				= (if(Nbootstraps>0) CI50lower else NULL),
 				CI50upper				= (if(Nbootstraps>0) CI50upper else NULL),
